/* =================================================================================
 *  alberi.c
 * =================================================================================
 * Contiene l'implementazione di funzioni e procedure su alberi binari.
 *  Authors:     Emanuela Fachini  (fachini@di.uniroma1.it )
 *               Irene Finocchi    (finocchi@di.uniroma1.it)
 *  Last change: 10 Maggio 2005
 *  Changed by:  Irene Finocchi
*/

#include "alberi.h"
#include "limits.h"

/* Pre: l'albero che si costruisce e' NON vuoto    */
/* Post: costruisce in preordine un albero binario */
TreePtr creaalbero() {
	TreePtr T;
	int i,j;

	scanf("%d %d",&i,&j);
	if (i==-1 && j==-1) return NULL;
	T = malloc(sizeof(TREENODE));

	if (T==NULL) printf("Impossibile allocare memoria per l'albero\n"); 
	else {
		T->elem=i;
		switch(j) {
			case 0:  T->rPtr=NULL;
				     T->lPtr=NULL;
				     break;
			case -1: T->lPtr=creaalbero();
				     T->rPtr=NULL;
				     break;
			case 1:  T->lPtr=NULL;
				     T->rPtr=creaalbero();
				     break;
			case 2:  T->lPtr=creaalbero();
				     T->rPtr=creaalbero();
				     break;
		}
	}
	return(T);
}


/* Post: stampa in inorder i valori contenuti nei nodi dell'albero*/
void stampaAlberoInorder(TreePtr T) {
	if (T!=NULL) {
		stampaAlberoInorder(T->lPtr);
		printf("%d ",T->elem);
		stampaAlberoInorder(T->rPtr);
	}
	return;
}

/* OSSERVAZIONE: nel main chiamata con h a zero */
/* Post: stampa i valori dell'albero indentati  */
void stampaIndentata (TreePtr T, int h) {
	int i;
	if (T!=NULL) {
		stampaIndentata(T->rPtr,h+1);
		for (i=1;i<=h;i++)
			printf("\t");
		printf("%d \n", T->elem);
		stampaIndentata(T->lPtr,h+1);
	}
	
}


/* Esercizio 18 */

int ABRaus (TreePtr tPtr, int *prec);

/* Verifica se l'albero soddisfa la proprieta' di ricerca */
/* PostC: restituisce 1 se tPtr punta alla radice di un albero binario di ricerca, 0 altrimenti */
int AlberoDiRicerca(TreePtr T) {
	int m = INT_MIN;
	return ABRaus(T,&m);
}


/*  PostC: restituisce 1 se tPtr punta alla radice di un albero binario di ricerca, 0 altrimenti */
/*  Lavora verificando che ogni intero nell'albero tPtr sia >= (*prec) */
/*  All'uscita dalla chiamata (*prec) e' il valore del massimo intero nell'albero tPtr */
int ABRaus (TreePtr tPtr, int *prec) {
	int ris;
	if (!tPtr) return 1;
	ris = ABRaus(tPtr->lPtr,prec);
	printf("Prec = %d,  Nodo = %d\n", *prec, tPtr->elem);
	ris = ris && (*prec) <= tPtr->elem;
	*prec = tPtr->elem;
	ris = ris && ABRaus(tPtr->rPtr,prec);
	return ris;
} 


/* Esercizio 19 */ 

/* Restituisce un puntatore al nodo i-esimo dell'albero t
   Prec:  t e' un albero binario di ricerca 
   Postc: restituisce il puntatore all'i-esimo nodo, se 1<=i<= num nodi, NULL altrimenti 
		  decrementa i del numero di nodi incontrati nell'albero radicato in t 
*/ 
TreePtr TrovaIesimo( TreePtr t, int *i) {
	TreePtr tempPtr;

	if (!t) return NULL;

	printf("i=%d  nodo=%d\n",*i,t->elem);
	
	tempPtr = TrovaIesimo(t->lPtr ,i);
	if (tempPtr) return tempPtr;

	if (*i == 1 ) return t;
	(*i)--;
	printf("  i decrementato a %d sul nodo %d\n",*i,t->elem);

	return TrovaIesimo(t->rPtr ,i);
}


/* Esercizio 20 */

/* Stampa in ordine crescente gli elementi dell'albero compresi tra i e j, estremi compresi
*  Prec:  i <= j */ 
void  compresi(TreePtr t,int i,int j) { 
	if (t != NULL) { 
		if (t ->elem > j)			/* gli elementi cercati non possono trovarsi nel sottoalbero destro */
			compresi(t->lPtr,i,j); 
		else if (t ->elem < i)		/* gli elementi cercati non possono trovarsi nel sottoalbero sinistro */
				compresi(t->rPtr,i,j);
		else {
			compresi(t->lPtr,i,j);
			printf("%d\n",t->elem);
			compresi(t->rPtr,i,j);
		}
	}
 }


